#--------------------------------------------------------------------
#----- GRNOC TL1 Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module supports the ability to retrieve data from several
#----- different models of TL1 devices.  Explictly supported devices
#----- include the following:
#-----
#----- * Cisco ONS15327
#----- * Cisco ONS15454
#----- * Cisco ONS15808
#----- * Nortel OME 6500
#----- * Nortel HDXc
#----- * Ciena CoreDirector
#----- * Infinera DTC
#----- * Fujitsu FLASHWAVE 7500
#----- 
#----- Each specifically supported device has its own
#----- GRNOC::TL1::Device module, which sets the default port and
#----- prompt used for that device.  They also may each export their
#----- own unique commands on top of what is already provided in
#----- GRNOC::TL1::Device.  Raw commands and output can be sent and
#----- received, or output can be parsed via the parse function, or
#----- by calling a function for that device.
#--------------------------------------------------------------------

package GRNOC::TL1;

use GRNOC::TL1::Parser;
use Net::Telnet;
use Data::Dumper;

use strict;

our $VERSION = "1.2.2";

sub new {
  
  my $caller = shift;
  my $class = ref($caller) || $caller;
  my $self = {
	      username => '',
	      password => '',
	      type => '',
	      device => '',
	      ctag => 1337,
	      tid => '',
	      prompt => '',
	      host => 'localhost',
	      port => '',
	      timeout => 10,
	      output_log => '',
	      input_log => '',
	      telnet => '',
	      parser => '',
	      wait_for_login_prompt => '',
	      debug => 0,
	      slow => 0,
	      @_
	     };
  
  bless($self, $class);

  # instantiate telnet object used for communication to TL1 gear
  $self->{'telnet'} = Net::Telnet->new();
  $self->{'telnet'}->host($self->{'host'});
  $self->{'telnet'}->timeout($self->{'timeout'});
  $self->{'telnet'}->errmode('return');  

  # setup input/output log files if given
  $self->{'telnet'}->input_log($self->{'input_log'})
    if ($self->{'input_log'} ne "");
  $self->{'telnet'}->output_log($self->{'output_log'})
    if ($self->{'output_log'} ne "");  
  
  # instantiate parser object to help parse TL1 output if needed
  $self->{'parser'} = GRNOC::TL1::Parser->new(debug => $self->{'debug'})
    if ($self->{'parser'} eq "");
  
  # determine which specific GRNOC::TL1::Device type to use, if any
  if ($self->{'type'} =~ /HDXc/i) {
    
    $self->{'device'} = GRNOC::TL1::Device::Nortel::HDXc->new(username => $self->{'username'},
							      password => $self->{'password'},
							      port => $self->{'port'},
							      ctag => $self->{'ctag'},
							      tid => $self->{'tid'},
							      telnet => $self->{'telnet'},
							      parser => $self->{'parser'},
							      debug => $self->{'debug'},
							      prompt => $self->{'prompt'},
							      wait_for_login_prompt => $self->{'wait_for_login_prompt'},
							      slow => $self->{'slow'});
  }

  elsif ($self->{'type'} =~ /OME\s*6500/i || $self->{'type'} =~ /OME/i) {
    
    $self->{'device'} = GRNOC::TL1::Device::Nortel::OME6500->new(username => $self->{'username'},
								 password => $self->{'password'},
								 port => $self->{'port'},
								 ctag => $self->{'ctag'},
								 tid => $self->{'tid'},
								 telnet => $self->{'telnet'},
								 parser => $self->{'parser'},
								 debug => $self->{'debug'},
								 prompt => $self->{'prompt'},
								 wait_for_login_prompt => $self->{'wait_for_login_prompt'},
								 slow => $self->{'slow'});
  }

  elsif ($self->{'type'} =~ /ONS\s*15327/i) {
              
    $self->{'device'} = GRNOC::TL1::Device::Cisco::ONS15327->new(username => $self->{'username'},
                                                                 password => $self->{'password'},
                                                                 port => $self->{'port'},
                                                                 ctag => $self->{'ctag'},
                                                                 tid => $self->{'tid'},
                                                                 telnet => $self->{'telnet'},
                                                                 parser => $self->{'parser'},
                                                                 debug => $self->{'debug'},
                                                                 prompt => $self->{'prompt'},
                                                                 wait_for_login_prompt => $self->{'wait_for_login_prompt'},
                                                                 slow => $self->{'slow'});
  }
  
  elsif ($self->{'type'} =~ /ONS\s*15454/i) {

    $self->{'device'} = GRNOC::TL1::Device::Cisco::ONS15454->new(username => $self->{'username'},
								 password => $self->{'password'},
								 port => $self->{'port'},
								 ctag => $self->{'ctag'},
								 tid => $self->{'tid'},
								 telnet => $self->{'telnet'},
								 parser => $self->{'parser'},
								 debug => $self->{'debug'},
								 prompt => $self->{'prompt'},
								 wait_for_login_prompt => $self->{'wait_for_login_prompt'},
								 slow => $self->{'slow'});
  }
  
  elsif ($self->{'type'} =~ /ONS\s*15808/i) {
    
    $self->{'device'} = GRNOC::TL1::Device::Cisco::ONS15808->new(username => $self->{'username'},
								 password => $self->{'password'},
								 port => $self->{'port'},
								 ctag => $self->{'ctag'},
								 tid => $self->{'tid'},
								 telnet => $self->{'telnet'},
								 parser => $self->{'parser'},
								 debug => $self->{'debug'},
								 prompt => $self->{'prompt'},
								 wait_for_login_prompt => $self->{'wait_for_login_prompt'},
								 slow => $self->{'slow'});
  }
  
  elsif ($self->{'type'} =~ /Core\s*Director/i || $self->{'type'} =~ /Ciena/i) {
    
    $self->{'device'} = GRNOC::TL1::Device::Ciena::CoreDirector->new(username => $self->{'username'},
								     password => $self->{'password'},
								     port => $self->{'port'},
								     ctag => $self->{'ctag'},
								     tid => $self->{'tid'},
								     telnet => $self->{'telnet'},
								     parser => $self->{'parser'},
								     debug => $self->{'debug'},
								     prompt => $self->{'prompt'},
								     wait_for_login_prompt => $self->{'wait_for_login_prompt'},
								     slow => $self->{'slow'});
  }
  
  elsif ($self->{'type'} =~ /(DTC|DTN)/i || $self->{'type'} =~ /Infinera/i) {
    
    $self->{'device'} = GRNOC::TL1::Device::Infinera::DTC->new(username => $self->{'username'},
							       password => $self->{'password'},
							       port => $self->{'port'},
							       ctag => $self->{'ctag'},
							       tid => $self->{'tid'},
							       telnet => $self->{'telnet'},
							       parser => $self->{'parser'},
							       debug => $self->{'debug'},
							       prompt => $self->{'prompt'},
							       wait_for_login_prompt => $self->{'wait_for_login_prompt'},
							       slow => $self->{'slow'});
  }

  elsif ($self->{'type'} =~ /fujitsu/i || $self->{'type'} =~ /flash\s*wave/i ||
	 $self->{'type'} =~ /flash\s*wave.*7500/i) {

    $self->{'device'} = GRNOC::TL1::Device::Fujitsu::FLASHWAVE7500->new(username => $self->{'username'},
									password => $self->{'password'},
									port => $self->{'port'},
									ctag => $self->{'ctag'},
									tid => $self->{'tid'},
									telnet => $self->{'telnet'},
									parser => $self->{'parser'},
									debug => $self->{'debug'},
									prompt => $self->{'prompt'},
									wait_for_login_prompt => $self->{'wait_for_login_prompt'},
								        slow => $self->{'slow'});
  }
  
  # no specific device found, use generic one
  else {

    $self->{'device'} = GRNOC::TL1::Device->new(username => $self->{'username'},
						password => $self->{'password'},
						port => $self->{'port'},
						ctag => $self->{'ctag'},
						tid => $self->{'tid'},
						telnet => $self->{'telnet'},
						parser => $self->{'parser'},
						debug => $self->{'debug'},
					        prompt => $self->{'prompt'},
						wait_for_login_prompt => $self->{'wait_for_login_prompt'},
					        slow => $self->{'slow'});
  }
  
  return $self;
}

sub connect {

  my $self = shift;
  
  return $self->{'device'}->connect();
}

sub disconnect {

  my $self = shift;

  $self->{'device'}->disconnect();
}

sub login {

  my $self = shift;

  return $self->{'device'}->login();
}

sub cmd {

  my ($self, $cmd) = @_;

  return $self->{'device'}->cmd($cmd);
}

sub send {

  my ($self, $cmd) = @_;
  
  return $self->{'device'}->send($cmd);
}

sub recv {

  my ($self, $cmd, $ctag) = @_;

  return $self->{'device'}->recv($cmd, $ctag);
}

sub parse {

  my ($self, $cmd) = @_;

  return $self->{'device'}->parse($cmd);
}

sub parse_autonomous_messages {

  my ($self, $output) = @_;

  return $self->{'device'}->parse_autonomous_messages($output);
}

sub get_alarms {

  my $self = shift;

  return $self->{'device'}->get_alarms();
}

sub get_conditions {

  my $self = shift;

  return $self->{'device'}->get_conditions();
}

sub get_inventory {

  my $self = shift;

  return $self->{'device'}->get_inventory();
}

sub get_facilities {

  my $self = shift;

  return $self->{'device'}->get_facilities();
}

sub get_facility {

  my ($self, $aid, $payload) = @_;

  return $self->{'device'}->get_facility($aid, $payload);
}

sub get_oc3_interface {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_oc3_interface($aid);
}

sub get_oc3_interfaces {

  my $self = shift;

  return $self->{'device'}->get_oc3_interfaces();
}

sub get_oc12_interface {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_oc12_interface($aid);
}

sub get_oc12_interfaces {

  my $self = shift;

  return $self->{'device'}->get_oc12_interfaces();
}

sub get_oc48_interface {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_oc48_interface($aid);
}

sub get_oc48_interfaces {

  my $self = shift;

  return $self->{'device'}->get_oc48_interfaces();
}

sub get_oc192_interface {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_oc192_interface($aid);
}

sub get_oc192_interfaces {

  my $self = shift;

  return $self->{'device'}->get_oc192_interfaces();
}

sub get_oc768_interface {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_oc768_interface($aid);
}

sub get_oc768_interfaces {

  my $self = shift;

  return $self->{'device'}->get_oc768_interfaces();
}

sub get_cross_connects {

  my $self = shift;

  return $self->{'device'}->get_cross_connects();
}

sub get_ethernet_interface {

  my ($self, $aid) = @_;
  
  return $self->{'device'}->get_ethernet_interface($aid);
}

sub get_ethernet_interfaces {

  my $self = shift;
  
  return $self->{'device'}->get_ethernet_interfaces();
}

sub get_10gig_ethernet_interface {

  my ($self, $aid) = @_;
  
  return $self->{'device'}->get_10gig_ethernet_interface($aid);
}

sub get_10gig_ethernet_interfaces {

  my $self = shift;
  
  return $self->{'device'}->get_10gig_ethernet_interfaces();
}

sub get_optical_interfaces {

  my $self = shift;

  return $self->{'device'}->get_optical_interfaces();
}

sub get_optical_channels {

  my $self = shift;

  return $self->{'device'}->get_optical_channels();
}

sub get_optical_multiplex_sections {

  my $self = shift;

  return $self->{'device'}->get_optical_multiplex_sections();
}

sub get_optical_channel_performance {

  my ($self, $aid, $montype, $monlev, $locn, $direction, $tmper, $date, $time) = @_;

  return $self->{'device'}->get_optical_channel_performance($aid, $montype, $monlev, $locn, $direction, $tmper, $date, $time);
}

sub get_performance_data {

  my ($self, $aid) = @_;

  return $self->{'device'}->get_performance_data($aid);
}

sub get_network_element_type {

  my $self = shift;

  return $self->{'device'}->get_network_element_type();
}

sub get_time {

  my $self = shift;

  return $self->{'device'}->get_time();
}

sub get_autonomous_output {

  my ($self, $type, $begintime, $endtime) = @_;

  return $self->{'device'}->get_autonomous_output($type, $begintime, $endtime);
}

sub backup_database {

  my ($self, $ftp_user, $ftp_pass, $ftp_host, $remote_filename) = @_;

  return $self->{'device'}->backup_database($ftp_user, $ftp_pass, $ftp_host, $remote_filename);
}

sub set_slow_mode {

  my ($self, $slow) = @_;

  $self->{'slow'} = $slow;
  $self->{'device'}{'slow'} = $slow;
}

sub get_slow_mode {

  my $self = shift;

  return $self->{'slow'};
}

1;
