#--------------------------------------------------------------------
#----- GRNOC TL1 Device Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module is the base class for GRNOC::TL1 device
#----- functionality.  Other GRNOC::TL1::Device classes may extend
#----- this class to override or include the methods provided.
#--------------------------------------------------------------------

package GRNOC::TL1::Device;

use strict;

use Time::HiRes qw(sleep);
use Data::Dumper;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;
  my $self = {
	      @_
	     };
  
  bless($self, $class);

  # set prompt if one was not given, usually its >
  $self->{'prompt'} = '>' if ($self->{'prompt'} eq "");

  # set port if one was not given, 23 maybe?
  $self->{'port'} = 23 if ($self->{'port'} eq '');
  $self->{'telnet'}->port($self->{'port'});

  # set wait for login prompt option if one was not given, usually we do
  $self->{'wait_for_login_prompt'} = 1 if ($self->{'wait_for_login_prompt'} eq '');
  
  return $self;
}

sub connect {

  my $self = shift;

  my $telnet = $self->{'telnet'};
  my $debug = $self->{'debug'};
  
  print STDERR "Connecting...\n" if ($debug);
  return $telnet->open();
}

sub disconnect {

  my $self = shift;

  my $telnet = $self->{'telnet'};
  my $debug = $self->{'debug'};
  my $username = $self->{'username'};
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  print STDERR "Disconnecting...\n" if ($debug);
  my $cmd = "CANC-USER:$tid:$username:$ctag;";

  # send the CANC-USER as a command so we wait for its response before disconnecting
  $self->cmd($cmd);

  # we may or may not already be disconnected now (behavior differs on various devices)
  $telnet->close();
}

sub login {

  my $self = shift;

  my $username = $self->{'username'};
  my $password = $self->{'password'};
  my $telnet = $self->{'telnet'};
  my $parser = $self->{'parser'};
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  my $debug = $self->{'debug'};
  my $prompt = $self->{'prompt'};
  my $wait_for_login_prompt = $self->{'wait_for_login_prompt'};
  
  print STDERR "Logging in...\n" if ($debug);
  my $login_string = "ACT-USER:$tid:" . $username . ":$ctag" . "::$password;";

  print STDERR "login: wait_for_login_prompt: $wait_for_login_prompt\n" if ($debug);

  # first we must read until prompt, if option specified
  if ($wait_for_login_prompt) {
    
    my $regex = "/^$prompt/gm";
    print STDERR "login: waiting for $regex\n" if ($debug);
    $telnet->waitfor($regex);
  }
  
  my $output = $self->cmd($login_string);
  my @response = $self->parse($output);
  
  return $response[0]->{'completion_code'} eq "COMPLD";
}

# make use of send and recv to send command and retrieve output
sub cmd {

  my ($self, $cmd) = @_;
 
  # send command, determine ctag send, retrieve output for that ctag
  my $ctag = $self->send($cmd);
  my $output = $self->recv($cmd, $ctag);
  my @output_messages = $self->parse($output);

  my $result;
  
  foreach my $output_message (@output_messages) {

    my $sid = $output_message->{'sid'};
    my $time = $output_message->{'time'};
    my $date = $output_message->{'date'};
    my $completion_code = $output_message->{'completion_code'};

    $result .= "\t$sid $date $time\n";
    $result .= "M  $ctag $completion_code\n";

    my $responses = $output_message->{'responses'};
    
    foreach my $response (@$responses) {

      $result .= "\t$response\n";
    }
    
    $result .= "\n";
  }
  
  return $result;
}

# sends data to the TL1 gear, returns the ctag of the command sent
sub send {

  my ($self, $cmd) = @_;
  
  my $telnet = $self->{'telnet'};
  my $parser = $self->{'parser'};
  my $debug = $self->{'debug'};

  # parse the input message
  my $input_message = $parser->parse_input_message($cmd);
  print STDERR Dumper($input_message) if ($debug);
  my $ctag = $input_message->{'ctag'};

  # see if we need to send the command slowly :)
  my $slow = $self->{'slow'};

  if ($slow) {

    print STDERR "send: Sending $cmd (slowly..)\n" if ($debug);

    for (my $i = 0; $i < length($cmd) - 1; $i++) {

      # sleep for a split moment before printing the next char
      sleep(.2);

      # print the next char in our command
      $telnet->put(substr($cmd, $i, 1));
    }
 
    # print the last char of the command after waiting for a bit
    sleep(2);
    $telnet->print(substr($cmd, length($cmd) - 1, 1));
  }

  else {

    print STDERR "send: Sending $cmd\n" if ($debug);
    $telnet->print($cmd);
  }

  return $ctag;
}

# receive raw data until the first COMPLD/DENY output message for a ctag
sub recv {

  my ($self, $cmd, $ctag) = @_;

  my $telnet = $self->{'telnet'};
  my $prompt = $self->{'prompt'};
  my $debug = $self->{'debug'};

  my $buf;
  
  print STDERR "recv: ctag is $ctag\n" if ($debug);

  # continually read lines
  while (my $line = $telnet->getline()) {

    # ignore lines which is the command that was typed in
    next if ($line =~ /$cmd/);

    print STDERR "recv: Line is $line" if ($debug);

    $buf .= $line;
    
    # is this the final output message for this ctag?
    if ($line =~ /^\s*M\s+$ctag\s+(COMPLD|DENY)/) {
 
      # read until the next prompt
      my $regex = "/^$prompt/gm";
      print STDERR "Waiting for: $regex\n" if ($debug);
      my ($prematch, $match) = $telnet->waitfor($regex);
      print STDERR "Got: $prematch" if ($debug);
      print STDERR "Got: $match" if ($debug);
      
      $buf .= $prematch;
      return $buf;
    }
  }
}

# parse out the desired output messages
sub parse {

  my ($self, $output) = @_;

  my $parser = $self->{'parser'};
  my $debug = $self->{'debug'};
  
  my @output_messages = $parser->parse_output_messages($output);

  print STDERR Dumper(@output_messages) if ($debug);
  
  # return the output messages
  return @output_messages;
}

sub parse_autonomous_messages {

  my ($self, $output) = @_;

  my $parser = $self->{'parser'};
  my $debug = $self->{'debug'};

  my @autonomous_messages = $parser->parse_autonomous_messages($output);

  print STDERR Dumper(@autonomous_messages)
    if ($debug);

  return @autonomous_messages;
}

# get all responses from a list of output messages
sub get_responses {

  my ($self, @output_messages) = @_;

  my @responses;

  foreach my $output_message (@output_messages) {

    my $responses = $output_message->{'responses'};

    foreach my $response (@$responses) {

      push(@responses, $response);
    }
  }

  return @responses;
}

# prototype for get_alarms
sub get_alarms {

  my $self = shift;

  return ("This device does not support get_alarms().");
}

# prototype for get_conditions
sub get_conditions {

  my $self = shift;

  return ("This device does not support get_conditions().");
}

# prototype for get_inventory
sub get_inventory {

  my $self = shift;

  return ("This device does not support get_inventory().");
}

# prototype for get_facilities
sub get_facilities {

  my $self = shift;

  return ("This device does not support get_facilities().");
}

# prototype for get_facility
sub get_facility {

  my ($self, $aid, $payload) = @_;

  return ("This device does not support get_facility().");
}

# prototype for get_oc3_interface
sub get_oc3_interface {

  my ($self, $aid) = @_;

  return ("This device does not support get_oc3_interface().");
}

# prototype for get_oc3_interfaces
sub get_oc3_interfaces {

  my $self = shift;

  return ("This device does not support get_oc3_interfaces().");
}

# prototype for get_oc12_interface
sub get_oc12_interface {

  my ($self, $aid) = @_;

  return ("This device does not support get_oc12_interface().");
}

# prototype for get_oc12_interfaces
sub get_oc12_interfaces {

  my $self = shift;

  return ("This device does not support get_oc12_interfaces().");
}

# prototype for get_oc48_interface
sub get_oc48_interface {

  my ($self, $aid) = @_;

  return ("This device does not support get_oc48_interface().");
}

# prototype for get_oc48_interfaces
sub get_oc48_interfaces {

  my $self = shift;

  return ("This device does not support get_oc48_interfaces().");
}

# prototype for get_oc192_interface
sub get_oc192_interface {

  my ($self, $aid) = @_;

  return ("This device does not support get_oc192_interface().");
}

# prototype for get_oc192_interfaces
sub get_oc192_interfaces {

  my $self = shift;

  return ("This device does not support get_oc192_interfaces().");
}

# prototype for get_oc768_interface
sub get_oc768_interface {

  my ($self, $aid) = @_;

  return ("This device does not support get_oc768_interface().");
}

# prototype for get_oc768_interfaces
sub get_oc768_interfaces {

  my $self = shift;

  return ("This device does not support get_oc768_interfaces().");
}

# prototype for get_cross_connects
sub get_cross_connects {

  my $self = shift;

  return ("This device does not support get_cross_connects().");
}

# prototype for get_ethernet_interface
sub get_ethernet_interface {

  my ($self, $aid) = @_;
  
  return ("This device does not support get_ethernet_interface().");
}

# prototype for get_ethernet_interfaces
sub get_ethernet_interfaces {

  my $self = shift;
  
  return ("This device does not support get_ethernet_interface().");
}

# prototype for get_10gig_ethernet_interface
sub get_10gig_ethernet_interface {

  my ($self, $aid) = @_;
  
  return ("This device does not support get_10gig_ethernet_interface().");
}

# prototype for get_10gig_ethernet_interfaces
sub get_10gig_ethernet_interfaces {

  my $self = shift;
  
  return ("This device does not support get_10gig_ethernet_interfaces().");
}

# prototype for get_optical_interfaces
sub get_optical_interfaces {

  my $self = shift;

  return ("This device does not support get_optical_interfaces().");
}

# prototype for get_optical_channels
sub get_optical_channels {

  my $self = shift;

  return ("This device does not support get_optical_channels().");
}

# prototype for get_optical_multiplex_sections
sub get_optical_multiplex_sections {

  my $self = shift;

  return ("This device does not support get_optical_multiplex_sections().");
}

sub get_optical_channel_performance {

  my ($self, $aid, $montype, $monlev, $locn, $direction, $tmper, $date, $time) = @_;

  return ("This device does not support get_optical_channel_performance().");
}

# prototype for get_performance_data
sub get_performance_data {

  my ($self, $aid) = @_;

  return ("This device does not support get_performance_data().");
}

# prototype for get_network_element_type
sub get_network_element_type {

  my $self = shift;

  return ("This device does not support get_network_element_type().");
}

# prototype for get_time
sub get_time {

  my $self = shift;

  return ("This device does not support get_time().");
}

# prototype for get_autonomous_output
sub get_autonomous_output {

  my ($self, $type, $begintime, $endtime) = @_;

  return ("This device does not support get_autonomous_output().");
}

# prototype for backup_database
sub backup_database {

  my ($self, $ftp_user, $ftp_pass, $ftp_host, $remote_filename) = @_;

  return ("This device does not support backup_database().");
}

1;
