#--------------------------------------------------------------------
#----- GRNOC TL1 Infinera DTC Device Module
#-----
#----- Copyright(C) 2009 The Trustees of Indiana University
#--------------------------------------------------------------------
#----- $HeadURL: $
#----- $Id: $
#-----
#----- This module handles the functionality for the Infinera DTC/DTN
#----- TL1 device.  It inherts from the standard GRNOC::TL1::Device
#----- module.
#--------------------------------------------------------------------

package GRNOC::TL1::Device::Infinera::DTC;
use base "GRNOC::TL1::Device";

use Data::Dumper;
use strict;

sub new {

  my $caller = shift;
  my $class = ref($caller) || $caller;
  
  my (%args) = @_;

  # set default port used for DTC if one was not given
  $args{'port'} = 9090 if ($args{'port'} eq '');

  # set default prompt used for DTC if one was not given
  $args{'prompt'} = 'TL1>>' if ($args{'prompt'} eq '');

  # call the GRNOC::TL1::Device structure
  my $self = $class->SUPER::new(%args);

  bless($self, $class);
  
  return $self;
}

sub get_alarms {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-ALM-ALL:$tid" . "::$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @alarms;

  foreach my $response (@responses) {

    my $alarm;

    if ($response =~ /"([^,]*),([^:]*):([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^:]*):[^"]*"*([^"]*)[^"]*"/) {
      
      $alarm->{'aid'} = $1;
      $alarm->{'aid_type'} = $2;
      $alarm->{'not_code'} = $3;
      $alarm->{'cond_type'} = $4;
      $alarm->{'serv_affect'} = $5;
      $alarm->{'date'} = $6;
      $alarm->{'time'} = $7;
      $alarm->{'location'} = $8;
      $alarm->{'direction'} = $9;
      $alarm->{'description'} = $10;
      
      $alarm->{'description'} =~ s/\\//g;
      $alarm->{'time'} =~ s/-/:/g;
      
      push(@alarms, $alarm);
    }

    else {

      print STDERR "get_alarms: Error parsing alarm string: $response\n";
    }
  }
  
  return @alarms;
}

sub get_inventory {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-EQPT:$tid:ALL:$ctag;";

  # store our old prompt
  my $old_prompt = $self->{'prompt'};

  # modify prompt to handle some Infineras which dont return the prompt
  # at the end of a RTRV-EQPT
  $self->{'prompt'} = ';';

  my @output_messages = $self->parse($self->cmd($cmd));

  # restore old prompt
  $self->{'prompt'} = $old_prompt;

  my @responses = $self->get_responses(@output_messages);
  my @inventory;

  foreach my $response (@responses) {

    my $device;

    if ($response =~ /^"([^:]+):([^:]+):.*"$/) {

      $device->{'aid'} = $1;
      $device->{'type'} = $2;
      
      ($device->{'description'}) = $response =~ /LABEL=([^,]*),/;
      ($device->{'serial'}) = $response =~ /SERNO=([^,]*),/;
      ($device->{'clei'}) = $response =~ /CLEI=([^,]*),/;
      ($device->{'part_number'}) = $response =~ /PARTNO=([^,]*),/;
      ($device->{'date'}) = $response =~ /MFGDATE=([^,]*),/;   
      ($device->{'subtype'}) = $response =~ /INSTTYPE=([^,]*),/;
      ($device->{'software_version'}) = $response =~ /SWVERS=([^,]*),/;
      ($device->{'version'}) = $response =~ /HWVERS=([^,]*),/;
      ($device->{'firmware'}) = $response =~ /FWVERS="([^"]*)"/;
      
      my ($pst, $sst) = $response =~ /:(IS|OOS)-([^"]*)"$/;
      $device->{'state'} = "$pst-$sst";

      $device->{'description'} =~ s/\\//g;
      $device->{'description'} =~ s/"//g;

      push(@inventory, $device);
    }
    
    else {
      
      print STDERR "get_inventory: Error parsing response: $response\n";
    }
  }

  return @inventory;
}

sub get_cross_connects {

  my $self = shift;
  
  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-CRS:$tid:ALL:$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @cross_connects;

  foreach my $response (@responses) {

    my $cross_connect;

    if ($response =~ /^"([^,]+),([^:]+):([^:]*):.*:(.*)"$/) {

      $cross_connect->{'from_aid'} = $1;
      $cross_connect->{'to_aid'} = $2;
      $cross_connect->{'type'} = $3;
      $cross_connect->{'state'} = $4;

      ($cross_connect->{'rate'}) = $response =~ /PAYLOAD=([^,]*),/;
      ($cross_connect->{'name'}) = $response =~ /LABEL=([^,]*),/;
      ($cross_connect->{'description'}) = $response =~ /CKTID=([^,]*),/;
      ($cross_connect->{'description2'}) = $response =~ /CKTIDSUFFIX=([^,]*),/;
      
      push(@cross_connects, $cross_connect);
    }
    
    else {
      
      print STDERR "get_cross_connects: Error parsing response: $response\n";
    }
  }

  return @cross_connects;
}

sub get_optical_interfaces {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};
  
  my $cmd = "RTRV-BAND:$tid:ALL:$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);
  my @interfaces;

  foreach my $response (@responses) {

    my $band;

    if ($response =~ /^"([^:]*):BAND:LABEL=([^,]*),MAXOCGS=([^,]*),CHANPLAN=([^,]*),HISTSTATS=([^:]*):([^"]*)"$/) {

      $band->{'aid'} = $1;
      $band->{'type'} = "BAND";
      $band->{'description'} = $2;
      $band->{'max_ocgs'} = $3;
      $band->{'chanplan'} = $4;
      $band->{'hist_stats'} = $5;
      $band->{'state'} = $6;

      push(@interfaces, $band);
    }

    else {

      print STDERR "get_optical_interfaces: Error parsing response: $response\n";
    }
  }

  my $cmd = "RTRV-OCG:$tid:ALL:$ctag;";
  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);

  foreach my $response (@responses) {

    my $ocg;

    if ($response =~ /^"([^:]*):OCG:LABEL=([^,]*),.*"$/) {
      
      $ocg->{'aid'} = $1;
      $ocg->{'type'} = "OCG";
      $ocg->{'description'} = $2;
      
      ($ocg->{'provisioned_dlm'}) = $response =~ /.*PROVDLM=([^,]*),.*/;
      ($ocg->{'discovered_dlm'}) = $response =~ /.*DISCDLM=([^,]*),.*/;
      ($ocg->{'ocg_num'}) = $response =~ /.*OCGNUM=([^,]*),.*/;
      ($ocg->{'hist_stats'}) = $response =~ /.*HISTSTATS=([^:]*):.*/;
      ($ocg->{'rmtocg'}) = $response =~ /.*RMTOCG=([^,]*),.*/;
      ($ocg->{'tectrl'}) = $response =~ /.*TECTRL=([^,]*),.*/;
      ($ocg->{'auto_disc_state'}) = $response =~ /.*AUTODISCSTATE=([^,]*),.*/;
      ($ocg->{'state'}) = $response =~ /.*:([^"]*)"$/;

      push(@interfaces, $ocg);
    }
    
    else {
      
      print STDERR "get_optical_interfaces: Error parsing response: $response\n";
    }
  }

  return @interfaces;
}

sub get_autonomous_output {

  my ($self, $type, $begintime, $endtime) = @_;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-AO:$tid" . "::" . $ctag . ':::MSGTYPE=' . $type;

  # begintime is optional
  $cmd .= ",BEGINTIME=$begintime"
    if ($begintime);
  
  # endtime is optional
  $cmd .= ",ENDTIME=$endtime"
    if ($endtime);

  $cmd .= ";";

  # send the command
  $self->send($cmd);

  # recv the raw output from this command
  my $output = $self->recv($cmd, $ctag);

  # grab all the autonomous messages from the output
  my @messages = $self->parse_autonomous_messages($output);

  my @results;

  # go through each auto message
  foreach my $message (@messages) {

    # right now I'm only handling REPT ALM
    if ($message->{'verb'} =~ /^REPT ALM/) {
      
      # go through each response in this message
      my $responses = $message->{'responses'};
      
      foreach my $response (@$responses) {
	
	# "<AID>:<NTFCNCDE>,<CONDTYPE>,<SRVEFF>,[<OCRDAT>],[<OCRTM>],[<LOCN>],[<DIRN>]:[<CONDDESCR>]"
	if ($response =~ /^"([^:]*):([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^,]*),([^:]*):(.*)$/) {
	  
	  my $alarm;

	  $alarm->{'aid'} = $1;
	  $alarm->{'not_code'} = $2;
	  $alarm->{'cond_type'} = $3;
	  $alarm->{'serv_affect'} = $4;
	  $alarm->{'date'} = $5;
	  $alarm->{'time'} = $6;
	  $alarm->{'location'} = $7;
	  $alarm->{'direction'} = $8;
	  $alarm->{'description'} = $9;
	  
	  # get rid of /" from description
	  $alarm->{'description'} =~ s/\\//g;
	  $alarm->{'description'} =~ s/"//g;

	  $alarm->{'time'} =~ s/-/:/g;

	  push(@results, $alarm);
	}

	else {

	  print STDERR "get_autonomous_output: Error parsing response: $response\n";
	}
      }
    }
  }

  return @results;
}

sub get_time {

  my $self = shift;

  my $ctag = $self->{'ctag'};
  my $tid = $self->{'tid'};

  my $cmd = "RTRV-SYS:$tid" . "::" . "$ctag;";

  my @output_messages = $self->parse($self->cmd($cmd));
  my @responses = $self->get_responses(@output_messages);

  # only expecting one response
  my $response = $responses[0];

  my $time;

  if ($response =~ /^"::.*"$/) {

    ($time->{'date'}) = $response =~ /SYSDATE=([^,]+),/;
    ($time->{'time'}) = $response =~ /SYSTIME=([^,]+),/;

    my ($year, $month, $day) = split(/-/, $time->{'date'});
    my ($hour, $min, $sec) = split(/-/, $time->{'time'});

    $time->{'year'} = $year;
    $time->{'month'} = $month;
    $time->{'day'} = $day;
    
    $time->{'hour'} = $hour;
    $time->{'min'} = $min;
    $time->{'sec'} = $sec;

    return $time;
  }

  else {

    print STDERR "get_time: Error parsing response: $response\n";

    return undef;
  }
}

1;
